import { getViewportOffset } from "../utils/index";

var __autoAdjustment__events__ = [];

export default {
    methods: {
        /*
         * 自动调整浮层（不绑定事件）
         *
         * @method layerAdjustmentBind
         * @param  {Dom}        layerElement        浮层元素
         * @param  {Dom}        targetElement       控制元素
         * @param  {Number}     distance            浮层元素和控制元素的上下间距
         */
        layerAdjustmentOnce(layerElement, targetElement, distance) {
            const viewportOffset = getViewportOffset(targetElement);

            let layerLeftDistance = 0;
            let layerHeight = 0;
            let layerWidth = 0;

            if (layerElement) {
                if (layerElement.getBoundingClientRect) {
                    layerHeight = layerElement.getBoundingClientRect().height;
                    layerWidth = layerElement.getBoundingClientRect().width;
                } else {
                    layerHeight = layerElement.clientHeight;
                    layerWidth = layerElement.clientWidth;
                }
            }

            if (viewportOffset.bottom < layerHeight) {
                layerElement.style.top =
                    viewportOffset.top - layerHeight - distance + "px";
            } else {
                layerElement.style.top =
                    viewportOffset.top +
                    targetElement.clientHeight +
                    distance +
                    "px";
            }

            const { left: viewportLeft, right: viewportRight } = viewportOffset;

            layerLeftDistance = viewportLeft;

            // 右侧距离不够时，弹出层居左显示
            if (
                viewportRight <= layerWidth &&
                layerLeftDistance >= layerWidth
            ) {
                layerLeftDistance =
                    layerLeftDistance - layerWidth + targetElement.clientWidth;
            }

            layerElement.style.left = layerLeftDistance + "px";
        },

        /*
         * 滚动时自动调整浮层
         *
         * @method layerAdjustmentBind
         * @param  {Dom}        layerElement        浮层元素
         * @param  {Dom}        targetElement       控制元素
         * @param  {Number}     distance            浮层元素和控制元素的上下间距
         */
        layerAdjustmentBind(layerElement, targetElement, distance) {
            var handler = (e) => {
                setTimeout((x) => {
                    this.layerAdjustmentOnce(
                        layerElement,
                        targetElement,
                        distance,
                    );
                });
            };

            __autoAdjustment__events__.push(handler);

            document.addEventListener("scroll", handler);
            window.addEventListener("resize", handler);
        },
    },
    beforeDestroy() {
        document.removeEventListener("scroll", __autoAdjustment__events__);
        window.removeEventListener("resize", __autoAdjustment__events__);
    },
};
